
/* Petit programme pour développer les séquences relatives à la gestion d'une rampe
   analogique affichant en relatif la Capacité actuellement en cours de mesurage
   par rapport à la Capacité théorique d'une batterie neuve.
  
UTILISATION : Appliquer une tension comprise entre 0 et environ 12V sur les bornes de la batterie,
              le fusible étant enlevé de son support. Mettre à jour la valeur désirée pour la
              constante "Capacite_indiquee_sur_la_batterie". Faire varier la tension entre zéro et la
              limite correspondant à la caractéristique de la batterie. La rampe doit s'étaler
              linéairement sur l'amplitude totale en largeur de l'afficheur. */

#include <EEPROM.h>
#include <LiquidCrystal.h>

#define Capacite_indiquee_sur_la_batterie 22 // Valeur du module alimentation personnel.

//--------------------- Définition des broches d'E/S ------------------------
#define Pilotage_RETRO 13 // Pilotage du rétroéclairage.
#define BUZZER 2 // Pilotage du bruiteur.
#define LED_Verte 3 // Pilotage de la sortie synchronisation.
#define Entree_TENSION 1  // Entrée analogique A1 pour mesurer la tension Batterie.

LiquidCrystal MonEcran(8, 9, 4, 5, 6, 7);
// Définition des matrices de représentation avant Void SETUP().
// On peut en écrire autant que l'on veut.
byte Eegu[8] = {2,4,14,17,31,16,14,0}; // Caractère spécial "é".
byte degre[8] = {4,10,4,0,0,0,0,0}; // Caractère spécial "°".
byte Fleche_Haut[8] = {4,14,21,4,4,4,4,0}; // Flèche vers le haut.
byte Fleche_Bas[8] = {4,4,4,4,21,14,4,0}; // Flèche vers le bas.
byte Gminuscule[8] = {0,0,15,17,17,15,1,14};
byte Pminuscule[8] = {0,0,30,17,17,30,16,16};
byte Jminuscule[8] = {4,0,15,2,2,2,18,12};
byte Damier[8] = {21,10,21,10,21,10,21,0}; // Simple grille pour démonstration.
// Matrices pour tracer le ruban linéaire.
byte I[8] = {16,16,16,16,16,16,16,16};
byte II[8] = {24,24,24,24,24,24,24,24};
byte III[8] = {28,28,28,28,28,28,28,28};
byte IIII[8] = {30,30,30,30,30,30,30,30};
byte IIIII[8] = {31,31,31,31,31,31,31,31};

//----------------------- Constanteses du programme -------------------------
#define Correction_U_Batterie 1.015 // Corrige la valeur du pont diviseur pour U batterie.
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P. >>> MAXI 60.

//------------------------- variables du programme --------------------------
byte Ordre_du_BP, Mode_affichage_ecran, NB_echantillons, Compteur;
byte Heures = 0, Minutes = 0, Secondes = 0;
int Mesure_NUM_CTN, CAN_U_Batterie, INDEX_MAX;
int INDEX; // Pointeur des octets situés en EEPROM.
double Temperature;
float U_Batterie; // Tension U aux bornes pour calcul de I en fonction de R.
float U_sur_Fusible; // Tension U aux bornes du fusible.
float Courant_de_decharge; // Est calculé par le programme.
float Capacite, Capacite_theorique;

float VALEUR; // Devra être comprise entre 0 et 96.

void Creer_les_caracteres_textuels() {
  MonEcran.createChar(1,degre);       // Caractère spécial "°".
  MonEcran.createChar(2,Damier);      //  Position intrinsèquement disponible.
  MonEcran.createChar(3,Jminuscule); // "j" minuscule.
  MonEcran.createChar(4,Gminuscule);  // "g" minuscule.
  MonEcran.createChar(5,Pminuscule);  // "p" minuscule.
  MonEcran.createChar(6,Eegu);        // Caractère spécial "é".
  MonEcran.createChar(7,Fleche_Haut); // Flèche vers le haut.
  MonEcran.createChar(8,Fleche_Bas);}  // Flèche vers le bas.
  
void Creer_les_rubans_du_sablier() {  
  MonEcran.createChar(1,I); MonEcran.createChar(2,II); MonEcran.createChar(3,III);
  MonEcran.createChar(4,IIII); MonEcran.createChar(5,IIIII);}

void setup(){
  pinMode(Pilotage_RETRO, OUTPUT); digitalWrite(Pilotage_RETRO, HIGH);
  pinMode(BUZZER, OUTPUT); digitalWrite(BUZZER, LOW); pinMode(LED_Verte, OUTPUT);
  MonEcran.begin(16,2); Creer_les_rubans_du_sablier();
  Capacite_theorique = Capacite_indiquee_sur_la_batterie; }

void loop() {
  // Simuler une valeur de Capacité en utilisant Entree_TENSION jusqu'à 12V pour 50AH.
  // La valeur qui en résulte pour deux fois la CAN est d'environ 1200.
  // On limitera la valeur numérisée à ce seuil.
  CAN_U_Batterie = 2 * (analogRead(Entree_TENSION)); // Mesure analogique [0 à 1023].
  if (CAN_U_Batterie > 1200) CAN_U_Batterie = 1200;
  Capacite = map(CAN_U_Batterie, 0, 1200, 0, 5000); // Translate en capacité float.
  Capacite = Capacite / 100;
  VALEUR = Capacite * 96 / Capacite_theorique;
  Ecrire_ligne_du_haut(); MonEcran.print(Capacite,2); 
  Affiche_la_rampe_ANALOGIQUE(); delay(100);}

void Affiche_la_rampe_ANALOGIQUE() {
  byte SAV_VALEUR = VALEUR; // Ici pas besoin d'un float.
  /*================== Afficher les pavés entiers. ======================*/
  VALEUR = (VALEUR / 6) - 1; Ecrire_ligne_du_bas();
  for (byte I = 0 ; I < VALEUR ; I++) MonEcran.write(5);
  /*================== Afficher la fraction de pavé. ====================*/
  VALEUR = SAV_VALEUR % 6;
  switch (byte(VALEUR)) {
    case 1 : {MonEcran.write(1); break;}
    case 2 : {MonEcran.write(2); break;}
    case 3 : {MonEcran.write(3); break;}
    case 4 : {MonEcran.write(4); break;}
    case 5 : {MonEcran.write(5); break;} } }
  
void Ecrire_ligne_du_haut() {
  Efface_ligne(0); MonEcran.setCursor(0,0);}
  
void Ecrire_ligne_du_bas() {
  Efface_ligne(1); MonEcran.setCursor(0,1);} 

void Efface_ligne(byte Ligne) {// Zéro : Haut, 1 : Bas.
  if (Ligne ==0) MonEcran.setCursor(0,0); else MonEcran.setCursor(0,1);
  MonEcran.print("                ");}
