
/* Petit programme de base pour tester des séquences qui seront intégrées au projet.
   Intègre la gestion de l'afficher LCD ainsi que la procédure d'exploitation du
   clavier. Les déclarations de constantes et de variables ainsi que l'affectation des E/S
   est cumulée au fur et à mesure que les séquences testées sont au point. Ainsi il y a concordance
   entre ce petit démonstrateur et le logiciel complet. 
   */

#include <EEPROM.h>
#include <LiquidCrystal.h>

//--------------------- Définition des broches d'E/S ------------------------
#define Pilotage_RETRO 13 // Pilotage du rétroéclairage.
#define BUZZER 2 // Pilotage du bruiteur.
#define LED_Verte 3 // Pilotage de la sortie synchronisation.
#define Capteur_KY_013 3 // A3 affectée à la mesure de température.
#define Entree_TENSION 1  // Entrée analogique A1 pour mesurer la tension Batterie.
#define Entree_Fusible 2  // Entrée analogique A2 pour mesurer la DDP au fusible.

LiquidCrystal MonEcran(8, 9, 4, 5, 6, 7);
byte Eegu[8] = {2,4,14,17,31,16,14,0}; // Caractère spécial "é".
byte degre[8] = {4,10,4,0,0,0,0,0}; // Caractère spécial "°".
byte Fleche_Haut[8] = {4,14,21,4,4,4,4,0}; // Flèche vers le haut.
byte Fleche_Bas[8] = {4,4,4,4,21,14,4,0}; // Flèche vers le bas.
byte Gminuscule[8] = {0,0,15,17,17,15,1,14};
byte Pminuscule[8] = {0,0,30,17,17,30,16,16};
byte Jminuscule[8] = {4,0,15,2,2,2,18,12};

//----------------------- Constanteses du programme -------------------------
#define Version_PGM "Version 21-08-17"
#define Correction_U_Batterie 1.015 // Corrige la valeur du pont diviseur pour U batterie.
#define Correction_U_Fusible 1.018 // Corrige la valeur mesurée pour U Fusible.
#define R_Bloc_thermique 2.7  // Valeur résultante des trois résistances mises en parallèle.
#define Ucritique 11.0 // Tension en dessous de laquelle le processus de décharge sera terminé.
#define T_base_millisecondes 996  // Durée globale entre deux mesures exprimée en mS. (+3mS)
#define Seuil_pour_alerte 45 // Température qui déclenchera l'alerte thermique.
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P. >>> MAXI 60.

//------------------------- variables du programme --------------------------
boolean Affiche_C_et_Nb;
byte Ordre_du_BP, Mode_affichage_ecran, NB_echantillons, Compteur;
byte Heures = 0, Minutes = 0, Secondes = 0;
int Mesure_NUM_CTN, CAN_U_Batterie, INDEX_MAX;
int INDEX; // Pointeur des octets situés en EEPROM.
double Temperature;
float U_Batterie; // Tension U aux bornes pour calcul de I en fonction de R.
float U_sur_Fusible; // Tension U aux bornes du fusible.
float Courant_de_decharge; // Est calculé par le programme.
float Capacite = 0;

unsigned long Ancienne_valeur_Temps_Ecoule_depuis_RESET;

void setup(){
  pinMode(Pilotage_RETRO, OUTPUT);
  pinMode(BUZZER, OUTPUT); digitalWrite(BUZZER, LOW); pinMode(LED_Verte, OUTPUT);
  MonEcran.begin(16,2);
  MonEcran.createChar(1,degre); // Caractère spécial "°".
  MonEcran.createChar(2,Eegu);  // Caractère spécial "é".
  MonEcran.createChar(3,Fleche_Haut);  // Flèche vers le haut.
  MonEcran.createChar(4,Fleche_Bas);  // Flèche vers le bas.
  MonEcran.createChar(5,Gminuscule);  // "g" minuscule.
  MonEcran.createChar(6,Pminuscule); // "p" minuscule.
  MonEcran.createChar(7,Jminuscule); // "j" minuscule.
  MonEcran.setCursor(0,0); MonEcran.print(">>> Bonjour. <<<");
  digitalWrite(Pilotage_RETRO, HIGH);} // Allume le rétroéclairage.

void loop() {                                          
  // >>>>>>>>>>>>>>>>>>>>>>>>>> Traiter un éventuel B.P. <<<<<<<<<<<<<<<<<<<<<<<<<<
  Tester_le_BP_A0();
  if (Ordre_du_BP > 0) {}
  // >>>>>>>>>>>>> >>>>>>>> Interruption une fois par seconde <<<<<<<<<<<<<<<<<<<<<
  if (millis() > Ancienne_valeur_Temps_Ecoule_depuis_RESET)  {
      Ancienne_valeur_Temps_Ecoule_depuis_RESET = millis() + 1000; // Réarme le processus.
      digitalWrite(LED_Verte,!digitalRead(LED_Verte)); // Inverser l'état de la LED verte.
      Mesurer_le_debit_batterie();} }
      
void Mesurer_le_debit_batterie() {delay(500);} // Simule un long traitement.

void Attendre_un_BP() { // Attend un clic sur l'un des cinq B.P.
  Ordre_du_BP = 0;
  while (Ordre_du_BP == 0) {Tester_le_BP_A0(); delay(30);
         digitalWrite(LED_Verte,!digitalRead(LED_Verte));} } // Inverser l'état de la LED verte.

void Affiche_un_zero() {MonEcran.print("0");}

void Affiche_OUI_NON() {
  Ecrire_ligne_du_bas(); MonEcran.print("( ");MonEcran.write(3); MonEcran.print(":OUI  "); MonEcran.write(4);
  MonEcran.print(":NON )");}
  
void Ecrire_ligne_du_haut() {
  Efface_ligne(0); MonEcran.setCursor(0,0);}
  
void Ecrire_ligne_du_bas() {
  Efface_ligne(1); MonEcran.setCursor(0,1);} 

void Efface_ligne(byte Ligne) {// Zéro : Haut, 1 : Bas.
  if (Ligne ==0) MonEcran.setCursor(0,0); else MonEcran.setCursor(0,1);
  MonEcran.print("                ");}
  
void BIP(int Frequence) {// 4000 très aigü, 3000 très fort et 500 : Mode clavier.
  tone(BUZZER,Frequence); delay(100); noTone(BUZZER); digitalWrite(BUZZER, LOW);}

void Tester_le_BP_A0() { // La lecture sur A0 donne 0 ou 1023 maximum.
  // >>>>>>>>>>>>> Le B.P. est câblé en "Analogique" <<<<<<<<<<<<<<<<<<
  // Aucun B.P : Temps dans la procédure court et retour = 0,
  // RIGHT = 1, UP = 2, DOWN = 3, LEFT = 4 SELECT = 5.
  Ordre_du_BP = 0; // Aucun BP appuyé.
  unsigned int Moyenne; // Valeur moyenne obtenue par la CAN de A0.
  Moyenne = 1023;
  for (byte I = 0; I < NB_tests_antirebonds; I++)
      {if (analogRead(0) < 830) Moyenne = analogRead(0);}
  //-------- Si appui détecté procéder à la mesure. --------
  if (Moyenne < 830)
     {Moyenne = 0;
      for (byte I = 0; I < NB_tests_antirebonds; I++) Moyenne = Moyenne + analogRead(0);
      //------------- Attendre le relacher du B.P. -------------
      for (byte I = 0; I < NB_tests_antirebonds; I++) {if (analogRead(0) < 830) I = 0;}
      //---------------- traite l'ordre du B.P. ----------------  
      Moyenne = int(Moyenne / NB_tests_antirebonds);
      if (Moyenne < 50) Ordre_du_BP = 1;
      if ((Moyenne > 49) && (Moyenne < 175)) Ordre_du_BP = 2;
      if ((Moyenne > 174) && (Moyenne < 330)) Ordre_du_BP = 3;
      if ((Moyenne > 329) && (Moyenne < 522)) Ordre_du_BP = 4;
      if ((Moyenne > 521) && (Moyenne < 830)) Ordre_du_BP = 5; BIP(500);} }
      
